var cpb = {};

cpb.INFO_MESSAGES_TYPE = "messages";
cpb.ERROR_MESSAGES_TYPE = "errors";
cpb.WARNING_MESSAGES_TYPE = "warnings";


cpb.widgetMap = {};
cpb.sectionIds = new Array();
cpb.activeTabId = {};
cpb.uiContextId = null;
cpb.screenId = null;
cpb.savedMessages = {};
cpb.clearOtherMsgs = true;

/**
 * Registers a widget that is on the current screen.
 * @param widget the widget to register.
 */
cpb.registerWidget = function(widget) {
   cpb.widgetMap[widget.id] = widget;
   if (widget.type == 'SECTION') {
      cpb.addSectionId("section_" + widget.id);
   }
}

cpb.getWidget = function(widgetId) {
   return cpb.widgetMap[widgetId];
}

cpb.addSectionId = function(sectionId) {
   cpb.sectionIds[cpb.sectionIds.length] = sectionId; 
}

/**
 * Executes the specified widgetAction.
 * @param widgetAction the widget action to execute.
 */
cpb.performWidgetAction = function(widgetAction) {
   var widget = cpb.getWidget(widgetAction.widgetId);
   if (widget) {
      // retrieve the function property by name 
      var func = widget[widgetAction.type];
      
      if (func) {
         // call the function in the scope of the widget with the action as the first argument
         func.call(widget, widgetAction);         
      } else {
         // TODO: remove this later
//         alert("Widget '" + widget.id + "' does not support " + widgetAction.type + "()");
      }
   }
}

/**
 * Called to evaluate an expression and return the result.
 * 
 * @param expression  the String expression to evaluate
 * @param callbackFunction the function to call with the result
 */
cpb.evaluateExpression = function(fieldName, expression, updateContext, callbackFunction) {
   var f = document.forms[0];
   var params = getParamsExcludeTable(f);

   CPBService.evaluateExpression(
      fieldName,
      expression,
      updateContext,
      params, 
      callbackFunction
   );
}


/**
 * Called when a field value is changed (usually only when that
 * field has some <on-change> actions.
 * @param fieldName the name of the field that changed
 */
cpb.fieldValueChanged = function(fieldName) {
   var f = cpb.getForm();
   var params = getParamsExcludeTable(f);
   
   CPBService.processFieldValueChange(
      fieldName, params, 
      function(actions) {
      
         // While executing this batch of widget actions, 
         // don't clear messages that already exist.
         cpb.setClearOtherMessages(false);
         
         for (var i = 0; i < actions.length; i++) 
         {
            cpb.performWidgetAction(actions[i]);
         }
         
         // go back to the default
         cpb.setClearOtherMessages(true);
      }
   );
   
   cpb.setPageDirty();
}

cpb.setPageDirty = function() {
   if (window.SaveChangesConfirmation)
   {
      window.SaveChangesConfirmation.setPageDirty(true);
   } 
   else if (window.parent.SaveChangesConfirmation)
   {
      window.parent.SaveChangesConfirmation.setChildPageDirty(true);
   }
}

/** 
 * PRIVATE function
 * Browser independent way of retrieving a dom element by id
 * @param id   the id you're looking for
 * @param doc  the document to search (or empty for current doc)
 */
elementById = function(id, doc) {
   if (!doc) {
      doc = document;
   }
   if (doc.getElementById) { 
      return doc.getElementById(id); 
   } else {
      return doc.all[id]; 
   } 
}

cpb.elementById = elementById;

cpb.getElement = function(obj) {
   if (cpb.isString(obj)) {
      return cpb.elementById(obj);
   }
   return obj;
}

cpb.isString = function(obj) {
   return (obj.constructor === String);
}

/**
 * Sets the enabled state of the specified element (or id).
 * @param element  the id or element you're looking for
 * @param enabled  true to enable, false to disable
 */
cpb.setEnabled = function(element, enabled) {
   element = cpb.getElement(element);
   if ((element != null) && (typeof(element) != "undefined")) {
	  if (element instanceof Array) {
		 for (var i = 0; i < element.length; i++) {
	        element[i].disabled = !enabled;
		 }
	  } else {
         element.disabled = !enabled;
	  }
   }
}

/**
 * Sets the enabled state of a sequence of elements based on the specified id.
 * @param id   the id you're looking for
 * @param enabled  true to enable, false to disable
 */
cpb.setSequenceEnabled = function(id, enabled) {
   var index = 0;
   var element = elementById(id + "_" + (index++));
   while (element) {
      cpb.setEnabled(element, enabled);
      element = elementById(id + "_" + (index++));
   }
}

/**
 * Sets the enabled state of the specified element (or id).
 * @param element  the id or element you're looking for
 * @param enabled  true to show, false to hide
 */
cpb.setVisible = function(elementIdOrObj, visible) {
   
   var element = cpb.getElement(elementIdOrObj);
   if (element) {
      element.style.display = visible ? "" : "none";
   }
}

/**
 * Sets the visible state of a sequence of elements based on the specified id.
 * @param id   the id you're looking for
 * @param enabled  true to show, false to hide
 */
cpb.setSequenceVisible = function(id, visible) {
   var index = 0;
   var element = elementById(id + "_" + (index++));
   while (element) {
      cpb.setVisible(element, visible);
      element = elementById(id + "_" + (index++));
   }
}

/**
 * Sets the value of the specified element (or id).
 * @param fieldName the name of the field to set the value of
 * @param newValue  the new value
 */
cpb.setValue = function(fieldName, form, newValue) {
//alert("setValue() -- fieldName: " + fieldName + " newValue: " + newValue);
   // If the newValue is a PageModification, then let it handle the modifications.
   
   if (newValue && newValue.className && newValue.className == 'PageModification') {
      PageModification.handle(newValue);
   } else {
      form = cpb.getForm(form);
      var element = form[fieldName];
      if ((element != null) && (typeof(element) != "undefined")) {
//alert("fieldName: " + fieldName + "  type: " + element.type + "  typeof: " + typeof(element) + "  length: " + element.length);
         if (element.type) {  // a single input field
            var hidden = element.type == 'hidden';
            if (element.type == 'text' || hidden || element.type == "textarea")  {
               element.value = newValue == null ? "" : newValue;
            } else if (element.type == 'select-one') { 
               cpb.selectOption(element, newValue);
            }
         } else if (element.length) {  // an array of inputs
            if (element.length > 0) {
//alert("fieldName: " + fieldName + "  element[0].type: " + element[0].type + "  newValue: " + newValue);
               if (element[0].type == 'radio') {
                  cpb.selectRadio(form, fieldName, "" + newValue);   // convert newValue to string
               } else if (element[0].type == 'checkbox') {
                  cpb.selectCheckboxes(form, fieldName, eval(newValue));
               }
            }
         }
      }
   }   
}

/**
 * Replaces the options in a select with new options from
 * the specified list.
 * @param id  the id of the select element
 * @param options  an array of label/value objects
 */
cpb.setOptions = function(id, options) {
   // remember what the previous value was
   var selectedValue = cpb.getOptionValue(id);
//   alert("selectedValue for id, " + id + ", == " + selectedValue);
   
   // replace the old options with the new options
   DWRUtil.removeAllOptions(id);
   DWRUtil.addOptions(id, options, "value", "label");
   
   // re-select the prior value (if it exists)
   DWRUtil.setValue(id, selectedValue);
}

/**
 * Replaces a group of radios with new radios based on
 * options from the specified list.
 * @param id    the id of element that contains the radio elements
 * @param name  the common name of the radios
 * @param form  the form (or name of the form)
 * @param options  an array of label/value objects
 * @param inline  true to keep all the radios on the same line
 */
cpb.setRadios = function(id, name, form, options, inline) {
   form = cpb.getForm(form);
   
   if (options && options != null) {
      var parent = elementById(id);
      
      // remember what the previous value was
      var selectedValue = cpb.getSelectedRadioValue(form, name);
      
      var html = "";
      // create new radios
      for (var i = 0; i < options.length; i++) {
         html += "<input type=\"radio\" name=\"" + name + "\""
            + " id=\"" + name + "_" + i + "\""
            + " value=\"" + options[i].value + "\" />"
            + options[i].label 
            + (inline ? "&nbsp;" : "<br/>");
      }
      parent.innerHTML = html;
      
      // re-select the prior value (if it exists)
      if (!cpb.selectRadio(form, name, selectedValue) && options.length > 0) {
         elementById(name + "_0").checked = "true";
      };
   }
}

cpb.setCheckboxes = function(id, name, form, options, inline) {
   form = cpb.getForm(form);
   
   if (options && options != null) {
      var parent = elementById(id);
      
      // remember what the previous value was
      var selectedValuesArray = cpb.getSelectedCheckboxValues(form, name);
      
      var html = "";
      // create new checkboxes
      for (var i = 0; i < options.length; i++) {
         if (options[i].value == null || options[i].value.length == 0)
            continue;
         html += "<input type=\"checkbox\" name=\"" + name + "\""
            + " id=\"" + name + "_" + i + "\""
            + " value=\"" + options[i].value + "\" />"
            + options[i].label 
            + (inline ? "&nbsp;" : "<br/>");
      }
      parent.innerHTML = html;
      
      // re-select the prior value (if it exists)
      for (var i=0; i < selectedValuesArray.length; i++) {
         cpb.selectCheckbox(form, name, selectedValuesArray[i]);
      }
   }
}

/**
 * Expands/collapses all sections.
 */
cpb.setSectionsExpanded = function(expanded) {
   if(typeof(expanded) == 'undefined')
      expanded = true;
   
   setVisible("cpb_collapseAll", expanded);
   setVisible("cpb_expandAll", !expanded);
   
   for (var i = 0; i <= cpb.sectionIds.length; i++) {
      setSectionExpanded(cpb.sectionIds[i], expanded);
   }
}

/**
 * Gets a map of CPB parameters and values from the form.
 */
cpb.getCPBParams = function(form) {
   var cpbPrefix = "cpb_";    // prefix for CPB params
   return getFieldMap(form, 
      function(element) {
         return (element.type == 'hidden' && 
            (element.name.indexOf(cpbPrefix) == 0));
      });
}

/**
 * Selects the specified tab in a particular tab section.
 */
cpb.selectTab = function(tabSectionKey, tabId) {
   var selectedTabId = cpb.activeTabId[tabSectionKey];

   // hide the previously selected tab
   if (selectedTabId) {
      cpb.setVisible(selectedTabId + "TabContent", false);
   }
   
   // and show the newly selected tab
   cpb.setVisible(tabId + "TabContent", true);
   
   // remember which one is selected
   elementById(tabSectionKey + "_actTab").value = tabSectionKey + ":" + tabId;   
   cpb.activeTabId[tabSectionKey] = tabId;
}

/**
 * Returns the value of the selected option in a select.
 * @param select  the select element
 */
cpb.getSelectedOptionValue = function(select) {
   return select.options[select.selectedIndex].value; 
}

cpb.getOptionValue = function(id) {
   var value = elementById(id).value;
   if (value == null || value == "")
   {
      var hidden_name = id + "_hidden";
      var hidden = elementById(hidden_name);
//      alert("hidden for " + id + " == " + hidden);
      if (hidden)
         value = hidden.value;
//      alert("value from hidden " + hidden_name + " for field " + id + " == " + value);
   }
   return value;
}

/**
 * Selects the option in a select that has the specified value.
 * @param select  the select element
 * @param value   the value of the option to select
 */
cpb.selectOption = function(select, value) {
   select = cpb.getElement(select);
   var options = select.options;
   if (options.length <= 1 && value != null)
   {
//      alert("building hidden for field " + select.name + " with value " + value);
      var hidden_name = select.name + "_hidden";
      var hidden = document.createElement("input");
      hidden.type = "hidden";
      hidden.id = hidden_name;
      hidden.value = value;
      document.body.appendChild(hidden);
//      alert("hidden " + hidden_name + " created " + elementById(hidden_name));
   }
   else
   {
      if (value == null) {
         value = "";
      }
      for (var i=0; i < options.length; i++) {
         if (options[i].value == value) {
            if (!options[i].selected) {
               options[i].selected = true;
               if (select.onchange) select.onchange();
            }
            return;
         }
      }
   }
}

/**
 * Returns the value of the selected radio button or null if none selected.
 * @param form       the form containing the radiobuttons
 * @param radioName  the common name of all the radiobuttons
 */
cpb.getSelectedRadioValue = function(form, radioName) {
   form = cpb.getForm(form);
   
   var rbs = cpb.getFormElementArray(form, radioName);
   for (var i=0; i < rbs.length; i++) {
      if (rbs[i].checked) {
         return (rbs[i].value);
      }
   }
   return (null);
}

/**
 * Returns an array of selected checkbox values or an empty array if none selected.
 * @param form          the form containing the checkboxes
 * @param checkboxName  the common name of all the checkboxes
 */
cpb.getSelectedCheckboxValues = function(form, checkboxName) {
   form = cpb.getForm(form);
   var cbs = cpb.getFormElementArray(form, checkboxName);
   var array = new Array();
   for (var i=0; i < cbs.length; i++) {
      if (cbs[i].checked) {
         array[array.length] = cbs[i].value;
      }
   }
   return array;
}

/**
 * Selects the radio button that has the specified value.
 * @param form       the form containing the radiobuttons
 * @param radioName  the common name of all the radiobuttons
 * @param value      the value of the radio to select
 */
cpb.selectRadio = function(form, radioName, value) {
   var rbs = cpb.getFormElementArray(form, radioName);
   for (var i=0; i < rbs.length; i++) {
      if (rbs[i].value == value) {
         rbs[i].checked = true;
         return true;
      }
   }
   return false;
}

/**
 * Selects the checkbox that has the specified value.
 * @param form          the form containing the checkboxes
 * @param checkboxName  the common name of all the checkboxes
 * @param value         the value of the checkbox to select
 */
cpb.selectCheckbox = function(form, checkboxName, value) {
   var cbs = cpb.getFormElementArray(form, checkboxName);
   for (var i=0; i < cbs.length; i++) {
      if (cbs[i].value == value) {
         cbs[i].checked = true;
         return true;
      }
   }
   return false;
}

/**
 * Selects the checkbox that has the specified values.
 * @param form          the form containing the checkboxes
 * @param checkboxName  the common name of all the checkboxes
 * @param values        the value of the checkbox to select
 */
cpb.selectCheckboxes = function(form, checkboxName, values) {
   var cbs = cpb.getFormElementArray(form, checkboxName);
   for (var i=0; i < cbs.length; i++) {
      var checked = false;
      for (var j=0; j < values.length; j++) {
         if (cbs[i].value == values[j]) {
            checked = true;
            break;
         }
      }
      cbs[i].checked = checked;
   }
   return false;
}

/**
 * Returns a form object given a form name, index, or form object.
 * @param form  the form name, index, or form object
 */
cpb.getForm = function(form) {
   if (form === undefined)
   {
      return document.forms[0];
   }
   else
   {
      var type = typeof(form);
      
      // if the form index or name was passed
      if (type == "number" || type == "string") {
         form = document.forms[form];
      }
      return form;
   }
}

/**
 * Returns an array of form elements given an input name.
 * @param form          the form containing the elements
 * @param commonName    the common name of all elements in the group
 */
cpb.getFormElementArray = function(form, commonName, elType) {
   form = cpb.getForm(form);
   
   var els = form[commonName];
   if (els == null) {
      els = new Array(0);
   } else if (!els.length) {  // only found one, so make into an array
      els = new Array(els);
   }
   
   if (typeof(elType) != "undefined") {
      var newEls = new Array();
      var size = 0;
       
      for (var i=0; i < els.length; i++) {
         if (els[i].type == elType) {
            newEls[size++] = els[i];
         }
      }
      return newEls;
   }
   
   return els;
}

cpb.popupform = function(myform, windowname)
{
    if (! window.focus)
        return true;
    var width = 800;
    var height = 600;
    var top = (screen.availHeight - height) / 2;
    var left = (screen.availWidth - width) / 2;
    
    if (top < 0) {
       top = 0;
    }
    if (left < 0) {
       left = 0;
    }
    
    var newwin;
    try {
    	newwin = window.open('', windowname, 'height=' + height + ',width=' + width + ',left=' + left + ',top=' + top + ',scrollbars=yes,resizable=yes');
    	if (newwin) {
	        newwin.focus();
    	    myform.target=windowname;
    	}
        return newwin;
    } catch (err) {
    	if (newwin != null) {
    		newwin.close();
    	}
    	return null;
    }
}
cpb.invokeCustomAction = function(actionId, uiContextId, screenId, output)
{
   var form = cpb.getForm();
   
   //save original action here because the form tag is used for other things.
   var savedAction = form.action;
   var savedTarget = form.target;
   
   form.action = "InvokeCustomAction.do?method=invoke&actionId=" + actionId + 
   		"&cpb_uiContextID=" + uiContextId + "&cpb_screenId=" + screenId;
   var success = true;
   if (output === undefined || output == "window") {
	   success = cpb.popupform(form, "sasActionPopup");
   } else {
      form.target = "_self";
   }
   if (success) {
	   form.submit();
   }
   
 //restore to original action
   form.action = savedAction;
   form.target = savedTarget;
}

cpb.invokeCustomActionAsynch = function (actionId, uiContextId, screenId, output)
{
  // use DWR to call method
   var form = cpb.getForm();
   var result = CPBService.invokeCustomAction(actionId, uiContextId, screenId, getParamsExcludeTable(form), 
         function(resultObject) {
            var actions     = resultObject.widgetActionList;
            var contentType = resultObject.contentType;
            var content     = resultObject.content;
            
            if (actions != null) {
               for (var i = 0; i < actions.length; i++) {
                  cpb.performWidgetAction(actions[i]);
               }
            } 
            if (contentType != null) {
               contentType = contentType.toLowerCase();
               if (content != null) {
                  if (contentType == "text/javascript" && content != null) {
                     eval(content);
                  } else if (contentType == "text/html") {
                     alert(contentType + " has not been supported yet");
                  } else if (contentType == "text/plain") {
                     alert(contentType + " has not been supported yet");
                  } else {
                     alert(contentType + " has not been supported yet");
                  }
               }
            }
               
   });
   
}

getParamsExcludeTable = function(form) {
   return getFieldMap(form, 
      function(element) {
         return isInterestingFieldElement(element) &&
            !(element.type == 'hidden' && (element.name.indexOf("_TC") >= 0));
      });
}

/**
 * This function returns the values encoded in an HTML form as string properties in an object. 
 * Disabled form elements, buttons, and other non-value form elements are skipped. 
 * Multi-select elements are returned as an array of string values.
 *
 * @param {Form|String} form - the form from which to retrieve values
 * @param {function} elementTestFunction - the function to use for deciding if an element 
 *    should be included (leave blank for default implementation)
 */
function getFieldMap(form, elementTestFunction) {
   var f = cpb.getForm(form);
   if (!elementTestFunction) {
      elementTestFunction = isInterestingFieldElement;
   }
   
   var result = {};
   
   var els = form.elements;
   for (var i = 0; i < els.length; i++) {
      var element = form.elements[i];
      if (elementTestFunction(element)) {
         var t = element.type;
         if (t == 'radio' || t == 'checkbox') {
            if (element.checked) {
               setMapValue(result, element.name, element.value);
            }
         } else {
            setMapValue(result, element.name, element.value);
         }
      }
   }
   
   return result;
}

/**
 * This function returns true if the element is interesting enough to get it's value.
 * @param {Object} element - the form element to test
 */
function isInterestingFieldElement(element) {
   var t = element.type;
      
   if (t == 'hidden' ||
       t == 'radio' ||
       t == 'checkbox' ||
       t == 'select-one' ||
       t == 'select-multiple' ||
       t == 'text' ||
       t == 'textarea') 
   {
      return !t.disabled;
   }
    
   return false;
}

/**
 * For the named property in the map, set the value. If a value 
 * already exists and it is a string, convert the value to be an 
 * array of values. 
 * @param {Object} map - the map to add values to
 * @param {String} name - the name of the value
 * @param {String} value - the value
 */
function setMapValue(map, name, value) {
   var val = map[name];
   if (isString(val)){
      map[name] = val + "#,#" + value;
   } else {
      map[name] = value;
   }
}


/**
 * Returns true if the object is a string.
 * @param {Object} obj - the object to test
 */
function isString(obj) {
   return (typeof obj == "string" || obj instanceof String);
}

/**
 * Returns true if the object is an Array.
 * @param {Object} obj - the object to test
 */
function isArray(obj) {
   return (obj && obj instanceof Array || typeof obj == "array");
}

PageModification = {};

/**
 * This method is overloaded with the following signatures:
 *    PageModification.handle(pageModification);
 *    PageModification.handle(elementId, pageModification);
 */
PageModification.handle = function(elementId, pageMod)
{
   var pageModification;
   
   if (pageMod === undefined)
   {
      /*
       * user provided just the page modification object
       * and there is no element ID argument...
       */
      pageModification = elementId;
   }
   else
   {
      /*
       * user provided element ID and page modification arguments...
       */
      pageModification = pageMod;
      if (!pageModification.htmlMap)
      {
         pageModification.htmlMap = {};
         pageModification.htmlMap[elementId] = pageModification.html;
      }
   }

   if (pageModification.htmlMap)
   {
      for (var elementId in pageModification.htmlMap)
      {
         var element = document.getElementById(elementId);
         
         if (!element)
         {
            element = document.createElement("DIV");
         }
         
         if (element.style.display == 'none')
         {
            element.style.display = '';
         }
         
         element.innerHTML = pageModification.htmlMap[elementId];
      }
   }
   
   displayMessages(cpb.ERROR_MESSAGES_TYPE, pageModification.errorMessages, cpb.getClearOtherMessages());
   displayMessages(cpb.WARNING_MESSAGES_TYPE, pageModification.warningMessages, cpb.getClearOtherMessages());
   displayMessages(cpb.INFO_MESSAGES_TYPE, pageModification.infoMessages, cpb.getClearOtherMessages());
   
   // then execute any javascript
   if (pageModification.javaScript)
   {
      eval(pageModification.javaScript);
   }
}

cpb.getSavedMessages = function(type)
{
   return cpb.savedMessages[type];
}

cpb.clearSavedMessages = function(type)
{
   if (type)
   {
      delete cpb.savedMessages[type];
   }
   else
   {
      // delete the types that we know about...
      delete cpb.savedMessages[cpb.ERROR_MESSAGES_TYPE];
      delete cpb.savedMessages[cpb.WARNING_MESSAGES_TYPE];
      delete cpb.savedMessages[cpb.INFO_MESSAGES_TYPE];
   }
}

cpb.saveMessages = function(type, messages) {
   cpb.savedMessages[type] = cpb.mergeMessages(
         cpb.getSavedMessages(type), messages);
}

cpb.mergeMessages = function(existingMessages, newMessages)
{
   if (!existingMessages)
   {
      existingMessages = [];
   }
   
   if (newMessages)
   {
      for (var i = 0; i < newMessages.length; i++) 
      {
         existingMessages.push(newMessages[i]);
      }
   }
   
   return existingMessages;
}

/**
 * Returns a string with the leading and trailing white-space removed.
 * @param {String} str - the string to trim
 */
cpb.trim = function(str) {
   var i = 0, j = str.length;
   
   while (i < j && str.charAt(i) <= ' ')
   {
      i++;
   }
   
   while (j > i && str.charAt(j - 1) <= ' ')
   {
      j--;
   }
   
   return str.substring(i, j);
}

/**
 * Dynamically display messages.
 * @param {String} type - the type of message to display [errors|warnings|messages]
 * @param {Message[]} messages - an array of message objects
 * @param {boolean} clearOtherMsgs - if not false, then other messages of the same type will be cleared
 */
function displayMessages(type, messages, clearOtherMsgs) {
   // info messages only have one area for messages
   
   // add saved messages to given messages array...
   messages = cpb.mergeMessages(messages, cpb.getSavedMessages(type));
   
   var el = (cpb.INFO_MESSAGES_TYPE == type) 
      ? elementById(type + "-content")
      : elementById("dyna-" + type + "-content");

   if(el != null) {
      if (messages && (messages.length > 0)) {
         var content = "";
         for (var i = 0; i < messages.length; i++) {
            content += "<div>" + messages[i].text + "</div>";
         }
         
         if (clearOtherMsgs == false) {
            el.innerHTML += content;
         } else {
            el.innerHTML = content;
         } 
         
         // show the message div
         setVisible(type, true);
         
         // scroll to the top if any errors occurred
         if (cpb.ERROR_MESSAGES_TYPE == type) {
            if (Messages !== undefined) {
               Messages.focusErrors();
            }
            else {
               window.scroll(0,0);
            }
         }
      }
      else if (clearOtherMsgs != false)
      {
         el.innerHTML = "";   // clear the dynamic msgs content
         
         // also hide the whole messages box if the static area is empty
         var staticMsgs = elementById(type + "-content");
         if (cpb.trim(staticMsgs.innerHTML) == "") {
            setVisible(type, false);
         }
      }
   }
}

cpb.displayMessages = displayMessages;

function common_handleWindowOnLoad()
{
   if (window.dwr !== undefined)
   {
      dwr.engine.setErrorHandler(dwrErrorHandler);
   }
}

function setEnabled(elementId, enabled) {
   var element = elementById(elementId);
   if ((element != null) && (typeof(element) != "undefined")) {
       element.disabled = !enabled;
   }
}

function setVisible(elementId, visible) {
   var element = elementById(elementId);
   if ((element != null) && (typeof(element) != "undefined")) {
       element.style.display = visible ? "" : "none";
   }
}

function setCollapsible(elementId, toggleImgId, afterMethod, toggleHrefId) {
   var element = elementById(elementId);
   var toggleImg = elementById(toggleImgId);
   
   toggleImg.targetId = elementId;
  var hidden = (element.style.display == "none");
  setToggleImage(toggleImg, hidden);
  toggleImg.style.cursor = "pointer";
  
  if (afterMethod != null) {
     toggleImg.afterMethod = afterMethod;
  }
  
  if (toggleHrefId) {
     var toggleHref = elementById(toggleHrefId);
     toggleHref.toggleImgId = toggleImgId;
     toggleHref.onclick = function() {
        var toggleImg = elementById(this.toggleImgId);
        toggleCollapsed(elementById(toggleImg.targetId), toggleImg);
        return false;         
     }; 
      toggleHref.onkeydown = function(evt) {
        var keycode;
        if (window.event) keycode = window.event.keyCode;
        else if (evt) keycode = evt.which;
        if (keycode == 37 || keycode == 39) { // left or right arrow
           setExpanded(elementId, toggleImgId, keycode == 39);
        }
     };  
  } else {
     toggleImg.onclick = function() {
        toggleCollapsed(elementById(this.targetId), this);
        return false;
     }; 
  }
  
}

//function setToggleImage(toggleImg, collapsed) {
//  if(collapsed) {
//     toggleImg.src   = "<fmt:message key='application.plus.image'/>";
//     toggleImg.alt   = "<fmt:message key='application.collapsableDiv.plus.tooltip.txt'/>";
//     toggleImg.title = "<fmt:message key='application.collapsableDiv.plus.tooltip.txt'/>";
//  } else {
//     toggleImg.src   = "<fmt:message key='application.minus.image'/>";
//     toggleImg.alt   = "<fmt:message key='application.collapsableDiv.minus.tooltip.txt'/>";
//     toggleImg.title = "<fmt:message key='application.collapsableDiv.minus.tooltip.txt'/>";
//  }
//}

function toggleCollapsed(element, toggleImg) {
   var hidden = (element.style.display == "none");
   element.style.display = hidden ? "" : "none";
   setToggleImage(toggleImg, !hidden);
       
   if (toggleImg.afterMethod) {
       toggleImg.afterMethod(hidden);
   }
}
function setExpanded(elementId, toggleImgId, expanded) {
  var element = elementById(elementId)
  var toggleImg = elementById(toggleImgId);
  if (element && toggleImg) {
     var hidden = (element.style.display == "none");
     if (hidden == expanded) {
        setVisible(elementId, expanded);
        setToggleImage(toggleImg, !expanded);
        if (toggleImg.afterMethod) {
           toggleImg.afterMethod(expanded);
        }
     }
  }
}

function setSectionExpanded(sectionId, expanded) {
  setExpanded(sectionId + '_body', sectionId + '_toggle_img', expanded);
}

/**
 * Sets the focus to the first input field on the page.
 */
cpb.setFocus = function() {
    for (var formIdx = 0; formIdx < document.forms.length; formIdx++) {
        var form = document.forms[formIdx];
      for (var i = 0; i < form.elements.length; i++) {
            var element = form.elements[i];
            if (element.disabled == true) continue;
            if (element.type == 'radio' && element.checked &&
                isVisible(element))
            {
                element.focus();
                break;
            } else if (element.focus &&
                element.type != 'hidden' && 
                element.type != 'button' &&
                element.type != 'submit' &&
                element.type != 'reset' &&
                isVisible(element))
            {
                try {
                    element.focus();
                } catch (error) {
                    // eat exception if field isn't visible
                }
                break;
            }
        }
    }
}

/**
 * Returns true if the specified element (and its ancestors) are visible.
 */
function isVisible(element) {
    while (element) {
        if (element.style && element.style.display == "none") return false;
        element = element.parentNode;
    }
    return true;
}

/**
 * Sets the clearOtherMessages flag.
 */
cpb.setClearOtherMessages = function(clearOtherMsgs)
{
   this.clearOtherMsgs = clearOtherMsgs;
}

/**
 * Retrieves the clearOtherMessages flag.
 */
cpb.getClearOtherMessages = function()
{
   return this.clearOtherMsgs != false;
}
